import visa
import sys
import time
from subprocess import call

GPIB_ADDRESS = 'GPIB0::18'
GNUPLOT = 'C:\\Program Files (x86)\\gnuplot\\bin\\gnuplot.exe'
PUTTY = 'C:\\s\\kitty.exe'
TARGET = 'admin@192.168.42.84'
DIRECTORY = 'C:\\XR3-No-Filters-Broadband\\'
FREQ_FIRST = 5100
FREQ_LAST = 5900
FREQ_STEP = 5
POWER_FIRST = 20
POWER_LAST = 20
POWER_STEP = 1
DWELL_SEC = 10

def readTraceData(instrument, trace):
	raw = instrument.ask("S2 TS O3 T{0}".format(trace))
	instrument.write("S1")
	tmp = ""
	for datum in raw:
		if datum == '\r':
			continue
		else:
			tmp += datum
	data = []
	for datum in tmp.split('\n'):
		data.append(float(datum))
	return data

class InstrumentAnnotations:
	def __init__(self, strings):
		self.battery			= strings[0]
		self.corrected			= strings[1]
		self.rbw				= strings[2]
		self.vbw				= strings[3]
		self.sweep				= strings[4]
		self.atten				= strings[5]
		self.ref				= strings[6]
		self.scale				= strings[7]
		self.trace_detection	= strings[8] # ???
		self.center_or_start	= strings[9]
		self.span_or_stop		= strings[10]
		self.ref_offset			= strings[11] # ???
		self.display_line		= strings[12]
		self.threshold			= strings[13] # ???
		self.marker_freq		= strings[14]
		self.marker_ampl		= strings[15]
		self.freq_offset		= strings[16] # ???
		self.video_avg			= strings[17]
		self.title				= strings[18]
		self.pl1_unlock			= strings[19]
		self.pl2_unlock			= strings[20]
		self.yto_unlock			= strings[21]
		self.het_unlock			= strings[22]
		self.m_n_unlock			= strings[23]
		self.ref_unlock			= strings[24]
		self.ext_oven			= strings[25]
		self.meas_uncal			= strings[26]
		self.freq_diag			= strings[27]
		self.UNUSED				= strings[28]
		self.srq				= strings[29]
		self.center_freq_step	= strings[30] # ???
		self.active_func		= strings[31]

	def __str__(self):
		return ("battery = {0}\n".format(self.battery) +
				"corrected = {0}\n".format(self.corrected) +
				"rbw = {0}\n".format(self.rbw) +
				"vbw = {0}\n".format(self.vbw) +
				"sweep = {0}\n".format(self.sweep) +
				"atten = {0}\n".format(self.atten) +
				"ref = {0}\n".format(self.ref) +
				"scale = {0}\n".format(self.scale) +
				"trace_detection = {0}\n".format(self.trace_detection) +
				"center_or_start = {0}\n".format(self.center_or_start) +
				"span_or_stop = {0}\n".format(self.span_or_stop) +
				"ref_offset = {0}\n".format(self.ref_offset) +
				"display_line = {0}\n".format(self.display_line) +
				"threshold = {0}\n".format(self.threshold) +
				"marker_freq = {0}\n".format(self.marker_freq) +
				"marker_ampl = {0}\n".format(self.marker_ampl) +
				"freq_offset = {0}\n".format(self.freq_offset) +
				"video_avg = {0}\n".format(self.video_avg) +
				"title = {0}\n".format(self.title) +
				"pl1_unlock = {0}\n".format(self.pl1_unlock) +
				"pl2_unlock = {0}\n".format(self.pl2_unlock) +
				"yto_unlock = {0}\n".format(self.yto_unlock) +
				"het_unlock = {0}\n".format(self.het_unlock) +
				"m_n_unlock = {0}\n".format(self.m_n_unlock) +
				"ref_unlock = {0}\n".format(self.ref_unlock) +
				"ext_oven = {0}\n".format(self.ext_oven) +
				"meas_uncal = {0}\n".format(self.meas_uncal) +
				"freq_diag = {0}\n".format(self.freq_diag) +
				"UNUSED = {0}\n".format(self.UNUSED) +
				"srq = {0}\n".format(self.srq) +
				"center_freq_step = {0}\n".format(self.center_freq_step) +
				"active_func = {0}\n".format(self.active_func))

def readAnnotations(instrument):
	raw = instrument.ask("OT")
	tmp = ""
	for char in raw:
		if char == '\r':
			continue
		else:
			tmp += char
	return InstrumentAnnotations(tmp.split('\n'))	

def printTraceData(data):
	strings = []
	for datum in data:
		strings.append(str(datum))
	print "\n".join(strings)

def generateGNUPlotGraph(data, annotations, directory, title):
	ref = float(annotations.ref.split(" ")[2])
	db_per = int(annotations.scale.split(" ")[0])
	str = ("set terminal pngcairo size 1003,740\n" +
		   "set output '{0}{1}.png'\n".format(directory, title) +
		   "set xrange [0:1000]\n" + # Not necessary, but included for readability
		   "set xtics 0, 100, 1000 format ''\n" +
		   "set yrange [{0}:{1}]\n".format(ref - (10 * db_per), ref) +
		   "set ytics {0}, {1}, {2} format ''\n".format(ref - (10 * db_per), db_per, ref) +
		   "set grid xtics front lt 1 lw 1 lc black\n" +
		   "set grid ytics front lt 1 lw 1 lc black\n" +
		   "set bmargin 2\n" +
		   "set tmargin 2\n" +
		   "set lmargin 0\n" +
		   "set rmargin 0.1\n" +
		   "set label \"{0}\" at character 1,1\n".format(annotations.center_or_start) +
		   "set label \"{0}\" at character 98,1\n".format(annotations.span_or_stop) +
		   "set label \"{0}\" at character 25,1\n".format(annotations.rbw) +
		   "set label \"{0}\" at character 51,1\n".format(annotations.vbw) +
		   "set label \"{0}\" at character 75,1\n".format(annotations.sweep) +
		   "set label \"{0}\" at character 1,38\n".format(annotations.ref) +
		   "set label \"{0}\" at character 15,38\n".format(annotations.scale) +
		   "set label \'\"{0}\"\' at character 25,38\n".format(title) +
		   "set label \"{0}\" at character 99,38\n".format(annotations.atten) +
		   "plot '{0}{1}.data' using 1 with lines notitle lw 1 lt rgb 'green'\n".format(directory, title))
	return str

def writePlotFiles(directory, title, data, commands):
	with open("{0}{1}.data".format(directory, title), 'w') as f:
		f.write("\n".join(map(str, data)))
	with open("{0}{1}.gnuplot".format(directory, title), 'w') as f:
		f.write(commands)

def renderScreen(gnuplot, directory, title, data, annotations):
	commands = generateGNUPlotGraph(data, annotations, directory, title)
	writePlotFiles(directory, title, data, commands)
	call([gnuplot, "{0}{1}.gnuplot".format(directory, title)])

def setPower(target, putty, directory, power):
	cmd_file = "{0}cmd.txt".format(directory)
	with open(cmd_file, 'w') as f:
		f.write("/interface wireless set 0 tx-power-mode=all-rates-fixed tx-power={0}\n".format(power))
	call([putty, target, '-m', cmd_file])
	time.sleep(0.1) # Allow the new power to settle before declaring it ready

def setFrequency(target, putty, directory, frequency):
	cmd_file = "{0}cmd.txt".format(directory)
	with open(cmd_file, 'w') as f:
		f.write("/interface wireless set 0 frequency={0}\n".format(frequency))
	call([putty, target, '-m', cmd_file])
	time.sleep(0.1) # Allow the new frequency to settle before declaring it ready

def resetMaxHold(instrument):
	instrument.write('A1 A2')

specan = visa.instrument(GPIB_ADDRESS)
annotations = readAnnotations(specan)

for power in range(POWER_FIRST, POWER_LAST + POWER_STEP, POWER_STEP):
	setPower(TARGET, PUTTY, DIRECTORY, power)
	for frequency in range(FREQ_FIRST, FREQ_LAST + FREQ_STEP, FREQ_STEP):
		print "+{0}dBm-at-{1}MHz ".format(power, frequency - 2000)
		setFrequency(TARGET, PUTTY, DIRECTORY, frequency)
		resetMaxHold(specan)
		time.sleep(DWELL_SEC) # Allow the max-hold to build up data
		data = readTraceData(specan, 'A')
		renderScreen(GNUPLOT, DIRECTORY, "+{0}dBm-at-{1}MHz".format(power, frequency - 2000), data, annotations)
